// file      : xsd/cxx-cli/parser/parser-header.cxx-cli
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005-2006 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#include <cxx-cli/parser/parser-header.hxx>

#include <xsd-frontend/semantic-graph.hxx>
#include <xsd-frontend/traversal.hxx>

#include <backend-elements/regex.hxx>

namespace CXX_CLI
{
  namespace Parser
  {
    namespace
    {
      //@@ TODO
      //
      struct List: Traversal::List, protected virtual Context
      {
        List (Context& c)
            : Context (c)
        {
        }

        virtual Void
        traverse (Type& l)
        {
          String name (l.name ());

          SemanticGraph::Type& t (l.argumented ().type ());

          //@@ What's to come is a nasty hack.
          //
          String item_name;

          if (dynamic_cast<SemanticGraph::Fundamental::IdRef*> (&t))
          {
            item_name = L"IDREF" == name ? L"IDREF_" : L"IDREF";
          }
          else if (dynamic_cast<SemanticGraph::Fundamental::IdRefs*> (&t))
          {
            item_name = L"IDREFS" == name ? L"IDREFS_" : L"IDREFS";
          }
          else
            item_name = t.name () == name ? t.name () + L"_" : t.name ();


          os << "template < typename " << id ("_xsd_", name) << ", "
             << "typename " << id ("_xsd_", item_name) << " >" << endl
             << "struct " << id (name)
             << ": ::xsd::cxx_cli::parser::list< " << id ("_xsd_", name) << ", "
             << id ("_xsd_", item_name) << ", " << char_type << " >"
             << "{"
             << "// You would normally override the pre, item," << endl
             << "// and post functions in your implementation."
             << "};";
        }
      };


      //@@ TODO
      //
      struct Union: Traversal::Union, protected virtual Context
      {
        Union (Context& c)
            : Context (c)
        {
        }

        virtual Void
        traverse (Type& u)
        {
          String name (u.name ());

          os << "template < typename " << id ("_xsd_", name) << " >" << endl
             << "struct " << id (name) <<
            ": ::xsd::cxx_cli::parser::traverser< " << id ("_xsd_", name) <<
            ", " << char_type << " >"
             << "{"
             << "// You would normally override the pre, _characters," << endl
             << "// and post functions in your implementation."
             << "};";
        }
      };


      //
      //
      struct BaseInheritance: Base
      {
        BaseInheritance (Context& c)
            : Context (c), Base (c)
        {
        }

        virtual Void
        traverse (Type& c)
        {
          os << ", ";
          Base::traverse (c);
        }
      };


      //
      //
      struct ParserHook : Traversal::Member,
                          protected virtual Context
      {
        ParserHook (Context& c)
            : Context (c)
        {
        }

        virtual Void
        traverse (Type& m)
        {
          String name (id (m.name ()));
          String type (id ("_xsd_", m.name ()));

          os << "virtual void" << endl
             << name << " (" << type << ");"
             << endl;
        }
      };


      //
      //
      struct ParserAccessor : Traversal::Member,
                              protected virtual Context
      {
        ParserAccessor (Context& c)
            : Context (c)
        {
        }

        virtual Void
        traverse (Type& m)
        {
          String name (id (m.name ()));
          String type (id ("_xsd_", m.name ()));

          os << "void" << endl
             << name << " (::xsd::cxx_cli::parser::Traverser< " <<
            type << " >^);"
             << endl;
        }
      };


      //
      //
      struct ParserMember : Traversal::Member,
                            protected virtual Context
      {
        ParserMember (Context& c)
            : Context (c)
        {
        }

        virtual Void
        traverse (Type& m)
        {
          String type (id ("_xsd_", m.name ()));
          String inst (id ("_xsd_", m.name (), "_"));

          os << "::xsd::cxx_cli::parser::Traverser< " <<
            type << " >^ " << inst << ";";
        }
      };


      //
      //
      struct Complex : Traversal::Complex,
                       protected virtual Context
      {
        Complex (Context& c)
            : Context (c),
              base_inheritance_ (c),
              template_param_decl_list_ (c),
              parser_hook_ (c),
              parser_member_ (c),
              parser_accessor_ (c)
        {
          inherits_ >> base_inheritance_;

          names_parser_hook_ >> parser_hook_;
          names_parser_member_ >> parser_member_;
          names_parser_accessor_ >> parser_accessor_;
        }

        virtual Void
        traverse (Type& c)
        {
          String name (id (c.name ()));
          String type (id ("_xsd_", c.name ()));
          Boolean has_members (has<Traversal::Element> (c) ||
                               has<Traversal::Attribute> (c));

          os << "generic < ";

          {
            template_param_decl_list_.traverse (c);
          }

          os << " >" << endl;

          //@@ Multiple inheritance not supported?
          //
          os << "public ref class " << name
             << ": ::xsd::cxx_cli::parser::Traverser< " << type << " >";

          {
            enter_class_scope (name, c);
            inherits<Complex> (c, inherits_, 0, &Complex::nl, &Complex::nl);
            leave_scope ();
          }

          os << "{"
             << "public:" << endl
             << endl;

          if (has_members)
          {
            os << "// Parser hooks. Override them in your implementation." << endl
               << "//" << endl
               << endl;

            enter_class_scope (name, c);
            names (c, names_parser_hook_);
            leave_scope ();
          }

          /*
            @@ Not legal in CLI.

          os << "using ::xsd::cxx_cli::parser::Traverser< " <<
            type << " >::pre;"
             << "using ::xsd::cxx_cli::parser::Traverser< " <<
            type << " >::post;"
             << endl
             << endl;
          */

          //
          //
          if (has_members)
          {
            os << "// Parser construction API." << endl
               << "//" << endl
               << endl;

            enter_class_scope (name, c);
            names (c, names_parser_accessor_);
            leave_scope ();

            //@@ TODO
            /*
            os << "void" << endl
               << "operator() (";

            {
              // Automatically enters the scope.
              //
              ParserParamDecl decl (*this, false);
              decl.traverse (c);
            }

            os << ");"
               << endl
               << endl;
            */
          }


          os << "// Implementation." << endl
             << "//" << endl
             << "public:" << endl
             << endl;


          // _start_element
          //
          os << "virtual void" << endl
             << "_start_element (" <<
            string_type << "^, " <<
            string_type << "^) override;"
             << endl;

          // end_element
          //
          os << "virtual void" << endl
             << "_end_element (" <<
            string_type << "^, " <<
            string_type << "^) override;"
             << endl;


          // attribute
          //
          os << "virtual void" << endl
             << "_attribute (" <<
            string_type << "^, " <<
            string_type << "^, " <<
            string_type << "^) override;"
             << endl;


          if (has_members)
          {
            os << "private:" << endl
               << endl;

            enter_class_scope (name, c);
            names (c, names_parser_member_);
            leave_scope ();
          }

          os << "};";
        }

        virtual Void
        nl (Type& c)
        {
          os << endl;
        }

      private:
        BaseInheritance base_inheritance_;
        Traversal::Inherits inherits_;

        //
        //
        TemplateParamDeclList template_param_decl_list_;

        //
        //
        ParserHook parser_hook_;
        Traversal::Names names_parser_hook_;

        //
        //
        ParserMember parser_member_;
        Traversal::Names names_parser_member_;

        //
        //
        ParserAccessor parser_accessor_;
        Traversal::Names names_parser_accessor_;
      };


      //
      //
      struct Includes : Traversal::Imports,
                        Traversal::Includes,
                        protected virtual Context
      {
        Includes (Context& c, String const& expr)
            : Context (c), expr_ (expr)
        {
        }

        virtual Void
        traverse (SemanticGraph::Imports& i)
        {
          traverse (i.file ());
        }

        virtual Void
        traverse (SemanticGraph::Includes& i)
        {
          traverse (i.file ());
        }

        virtual Void
        traverse (SemanticGraph::Path const& f)
        {
          using BackendElements::Regex::perl_s;

          // Note the use of the portable representation of the path.
          //
          String name (perl_s (String (f.string ()), expr_));

          Char op (include_with_brackets ? '<' : '"');
          Char cl (include_with_brackets ? '>' : '"');

          os << "#include " << op << include_prefix << name << cl << endl
             << endl;
        }

      private:
        String expr_;
      };


      struct FundamentalNamespace : Namespace,
                                    protected virtual Context
      {
        FundamentalNamespace (Context& c)
            : Context (c),
              Namespace (c)
        {
        }

        void
        traverse (Type& ns)
        {
          pre (ns);

          os << "// Parsers for XML Schema built-in types." << endl
             << "//" << endl
             << endl;

          /*

          os << "// 8-bit" << endl
             << "//" << endl
             << "typedef ::xsd::cxx_cli::parser::byte< " <<
            char_type << " > byte;"
             << "typedef ::xsd::cxx_cli::parser::unsigned_byte< " <<
            char_type << " > unsigned_byte;"
             << endl;


          os << "// 16-bit" << endl
             << "//" << endl
             << "typedef ::xsd::cxx_cli::parser::short_< " <<
            char_type << " > short_;"
             << "typedef ::xsd::cxx_cli::parser::unsigned_short< " <<
            char_type << " > unsigned_short;"
             << endl;


          os << "// 32-bit" << endl
             << "//" << endl
             << "typedef ::xsd::cxx_cli::parser::int_< " <<
            char_type << " > int_;"
             << "typedef ::xsd::cxx_cli::parser::unsigned_int< " <<
            char_type << " > unsigned_int;"
             << endl;


          os << "// 64-bit" << endl
             << "//" << endl
             << "typedef ::xsd::cxx_cli::parser::long_< " <<
            char_type << " > long_;"
             << "typedef ::xsd::cxx_cli::parser::unsigned_long< " <<
            char_type << " > unsigned_long;"
             << endl;

          os << "// Boolean." << endl
             << "//" << endl
             << "typedef ::xsd::cxx_cli::parser::boolean< " <<
            char_type << " > boolean;"
             << endl;

          os << "// Floating-point types." << endl
             << "//" << endl
             << "typedef ::xsd::cxx_cli::parser::float_< " <<
            char_type << " > float_;"
             << "typedef ::xsd::cxx_cli::parser::double_< " <<
            char_type << " > double_;"
             << "typedef ::xsd::cxx_cli::parser::decimal< " <<
            char_type << " > decimal;"
             << endl;
          */

          os << "// String." << endl
             << "//" << endl
             << "typedef ::xsd::cxx_cli::parser::String String;"
             << endl
             << endl;


          /*
          os << "// Exceptions. See libxsd/xsd/cxx-cli/parser/exceptions.hxx for " <<
            "details." << endl
             << "//" << endl
             << "typedef ::xsd::cxx_cli::parser::exception exception;"
             << "typedef ::xsd::cxx_cli::parser::invalid_instance invalid_instance;"
             << "typedef ::xsd::cxx_cli::parser::unexpected_element< " <<
            char_type << " > unexpected_element;"
             << endl;

          os << "// Parsing flags. See libxsd/xsd/cxx-cli/parser/elements.hxx for details." << endl
             << "//" << endl
             << "typedef ::xsd::cxx_cli::parser::flags flags;"
             << endl;

          os << "// Parsing properties. See libxsd/xsd/cxx-cli/parser/elements.hxx for details." << endl
             << "//" << endl
             << "typedef ::xsd::cxx_cli::parser::properties< " << char_type <<
            " > properties;"
             << endl;
          */

          post (ns);
        }
      };
    }

    Void
    generate_parser_header (Context& ctx,
                            SemanticGraph::Schema& tu,
                            String const& expr)
    {
      /*
      String c (ctx.char_type);

      if (c == L"char")
      {
        ctx.os << "#ifndef XSD_USE_CHAR" << endl
               << "#define XSD_USE_CHAR" << endl
               << "#endif" << endl
               << endl;
      }
      else if (c == L"wchar_t")
      {
      */

      ctx.os << "#ifndef XSD_USE_WCHAR" << endl
             << "#define XSD_USE_WCHAR" << endl
             << "#endif" << endl
             << endl;
      /*
      }
      */

      ctx.os << "#include <xsd/cxx-cli/parser/exceptions.hxx>" << endl
             << "#include <xsd/cxx-cli/parser/elements.hxx>" << endl
             << "#include <xsd/cxx-cli/parser/types.hxx>" << endl
             << endl;

      // Emit fundamental types.
      //
      {
        Traversal::Schema schema, xsd;
        Traversal::Implies implies;
        Traversal::Names names;
        FundamentalNamespace ns (ctx);

        schema >> implies >> xsd >> names >> ns;

        schema.dispatch (tu);
      }

      Traversal::Schema schema;

      Traversal::Sources sources;
      Includes includes (ctx, expr);
      Traversal::Names schema_names;

      Namespace ns (ctx);
      Traversal::Names names;

      schema >> includes;
      schema >> sources >> schema;
      schema >> schema_names >> ns >> names;

      List list (ctx);
      Union union_ (ctx);
      Complex complex (ctx);

      names >> list;
      names >> union_;
      names >> complex;

      schema.dispatch (tu);
    }
  }
}

